package rmidemo;

import java.util.Map;
import java.io.IOException;
import java.io.LineNumberReader;
import javax.swing.event.TableModelEvent;
import java.rmi.registry.LocateRegistry;
import java.rmi.registry.Registry;


public class DataModelImplTest {

   private DataModel model;

   public DataModelImplTest() {
      this(false);
   }

   public DataModelImplTest(boolean remote) {
      if (remote) {
	 try {
	    Registry registry = LocateRegistry.getRegistry();
	    DataModelFactory stub 
	       = (DataModelFactory) registry.lookup("DataModelFactory");
	    model = stub.getDataModel();
	 } catch (java.rmi.RemoteException e) {
	    throw new RuntimeException(e);
	 } catch (java.rmi.NotBoundException e) {
	    throw new RuntimeException(e);
	 }
      } else {
	 model = new DataModelImpl();
      }
      for (BasicCommand x : basic_commands)
	 commands.put(x.getCmd(),x);
      commands.put("exit",commands.get("quit"));

      model.addTableModelListener(new javax.swing.event.TableModelListener() {
	    public void tableChanged(TableModelEvent evt) {
	       String type = "unknown";
	       switch (evt.getType()) {
	       case TableModelEvent.INSERT: type="INSERT"; break;
	       case TableModelEvent.UPDATE: type="UPDATE"; break;
	       case TableModelEvent.DELETE: type="DELETE"; break;
	       }
	       System.err.println("Table model event: "+
				  "type="+type+" "+
				  "col="+evt.getColumn()+" "+
				  "row0="+evt.getFirstRow()+" "+
				  "rown="+evt.getLastRow());
				  
	    }
	 });
   }

   protected interface Command {
      int getMinArgs();
      void exec(String args[]);
   }
   private abstract class BasicCommand implements Command {
      abstract String getCmd();
   }
   
   private final BasicCommand[] basic_commands = {
      new BasicCommand() { 
	 String getCmd() { return "all"; }
	 public int getMinArgs() { return 0; }
	 public void exec(String[] args) {
	    int n = model.getRowCount();
	    int ncol = model.getColumnCount();
	    int i,j;
	    for (j=0;j<ncol;j++)
	       System.out.printf("%19s ",model.getColumnName(j));
	    System.out.println();
	    for (j=0;j<ncol;j++)
	       System.out.print("------------------- ");
	    System.out.println();
	    for (i=0;i<n;i++) {
	       for (j=0;j<ncol;j++)
		  System.out.printf("%19s ",model.getValueAt(i,j));
	       System.out.println();
	    }
	 }},
      new BasicCommand() {
	 String getCmd() { return "get"; }
	 public int getMinArgs() { return 2; }
	 public void exec(String args[]) {
	    System.out.println(model.getValueAt(Integer.parseInt(args[0]),
						Integer.parseInt(args[1])));
	 }},
      new BasicCommand() {
	 String getCmd() { return "set"; }
	 public int getMinArgs() { return 3; }
	 public void exec(String args[]) {
	    model.setValueAt(args[2],
			     Integer.parseInt(args[0]),
			     Integer.parseInt(args[1]));
	 }},
      new BasicCommand() {
	 String getCmd() { return "quit"; }
	 public int getMinArgs() { return 0; }
	 public void exec(String args[]) {
	    System.exit(0);
	 }},
      new BasicCommand() {
	 String getCmd() { return "filter"; }
	 public int getMinArgs() { return 1; }
	 public void exec(String args[]) {
	    model.setFilter(args[0]);
	 }},
      new BasicCommand() {
	 String getCmd() { return "del"; }
	 public int getMinArgs() { return 1; }
	 public void exec(String args[]) {
	    model.deleteRow(Integer.parseInt(args[0]));
	 }},      
   };

   Map<String,Command> commands 
      = new java.util.HashMap<String,Command>();   

   public void exec(String cmd, String args[]) {
      Command x = commands.get(cmd);
      if (x==null) {
	 throw new RuntimeException("No such command '"+cmd+"'");
      }
      if (args.length < x.getMinArgs()) {
	 throw new RuntimeException("Command '"+cmd+"' requires "+
				    "at least "+x.getMinArgs()+" arguments");
      }
      x.exec(args);	 
   }
   
		/** Contents of the Data Register. */ 
   static String[] cdr(String[] arr) {
      String[] dr = new String[arr.length-1]; 
      System.arraycopy(arr,1,dr,0,dr.length);
      return dr;
   }

   public static void main(String[] args) {
      String lr = "-local";
      if (args.length>0) {
	 args[0] = args[0].intern();
	 if (args[0]=="-local" || args[0]=="-remote") {
	    lr = args[0];
	    args = cdr(args);
	 }
      }
      DataModelImplTest modelTest;
      if (lr=="-local") {
	 modelTest = new DataModelImplTest();
      } else {
	 modelTest = new DataModelImplTest(true);
      }
      if (args.length > 0) {
	 String cmd = args[0];
	 args = cdr(args);
	 modelTest.exec(cmd,args);
      } else {
	 try {
	    LineNumberReader in = new LineNumberReader(new java.io.InputStreamReader(System.in));
	    String line;

	    prompt();
	    while ((line=in.readLine())!=null) {
	       try {
		  String[] a0 = line.split("\\s+",2);
		  String cmd = a0[0].toLowerCase();
		  if (a0.length==1)
		     args = new String[0];
		  else
		     args = a0[1].trim().split(",\\s*");
		  modelTest.exec(cmd,args);
	       } catch (RuntimeException e) {
		  e.printStackTrace();
	       }
	       prompt();
	    }
	 } catch (IOException e) {
	    throw new RuntimeException(e); 
	 }
      }
   }


   static void prompt() {
      System.out.print("DataModelImplTest % ");
      System.out.flush();
   }
}